@@ -1,5 +1,6 @@
 import json
 import logging
+from httpcache import CachingHTTPAdapter
 import types
 import urlparse
 import itertools
@@ -30,13 +31,7 @@ class SolrException(BaseException):
 
 
 class Solr(object):
-    _add_batch = []
-    _shards = None
-
-    endpoints = None
-    default_endpoint = None
-
-    def __init__(self, endpoints, default_endpoint=None):
+    def __init__(self, endpoints, default_endpoint=None, http_cache=True):
         if not endpoints:
             logger.warning("Faulty Solr configuration, SOLR will not be available!")
             return
@@ -45,6 +40,11 @@ class Solr(object):
         self.default_endpoint = None
         self._shards = None
         self._add_batch = list()
+        self.req_session = requests.Session()
+
+        if http_cache:
+            self.req_session.mount("http://", CachingHTTPAdapter())
+            self.req_session.mount("https://", CachingHTTPAdapter())
 
         if isinstance(endpoints, basestring):
             self.endpoints = {'default': endpoints}
@@ -64,7 +64,7 @@ class Solr(object):
         # Check document language and dispatch to correct core
         url = _get_url(core_url, "update")
         try:
-            response = requests.post(url, data=json_command, headers={'Content-Type': 'application/json'})
+            response = self.req_session.post(url, data=json_command, headers={'Content-Type': 'application/json'})
             response.raise_for_status()
         except requests.RequestException as e:
             logger.error("Failed to send update to Solr endpoint [%s]: %s", core_url, e, exc_info=True)
@@ -73,7 +73,7 @@ class Solr(object):
 
     def _send_solr_query(self, request_url, query):
         try:
-            response = requests.post(request_url, data=query)
+            response = self.req_session.post(request_url, data=query)
             response.raise_for_status()
             results = response.json()
         except requests.RequestException as e:
@@ -81,16 +81,20 @@ class Solr(object):
             return None
         return results
 
-    def add(self, documents):
+    def add(self, documents, boost=None):
         """
         Adds documents to Solr index
         documents - Single item or list of items to add
         """
 
-        if isinstance(documents, types.ListType):
-            self._add_batch.extend(documents)
-        else:
-            self._add_batch.append(documents)
+        if not isinstance(documents, types.ListType):
+            documents = [documents]
+        documents = [{'doc': d} for d in documents]
+        if boost:
+            for d in documents:
+                d['boost'] = boost
+
+        self._add_batch.extend(documents)
 
         if len(self._add_batch) > SOLR_ADD_BATCH:
             self._addFlushBatch()
@@ -104,8 +108,8 @@ class Solr(object):
             # Create command JSONs for each of language endpoints
             for lang in self.endpoints:
                 # Append documents with languages without endpoint to default endpoint
-                document_jsons = ["\"add\":{\"doc\": " + json.dumps(data) + "}" for data in self._add_batch
-                                  if data.get("language", self.default_endpoint) == lang or (lang == self.default_endpoint and not self.endpoints.has_key(data.get("language")))]
+                document_jsons = ["\"add\":" + json.dumps(data) for data in self._add_batch
+                                  if data['doc'].get("language", self.default_endpoint) == lang or (lang == self.default_endpoint and not self.endpoints.has_key(data['doc'].get("language")))]
                 command_json = "{" + ",".join(document_jsons) + "}"
                 language_batches[lang] = command_json
             # Solr requires for documents to be sent in { "add" : { "doc" : {...} }, "add": { "doc" : { ... }, ... }
